package fr.feufochmar.scribbly;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Map.Entry;

/**
 * A simple Scribbly document writer.
 * @author Feufochmar
 *
 */
public class ScribblyDocumentWriter implements ScribblyVisitor
{
  private final ScribblyDocument m_document;
  private Writer m_writer = null;
  
  /**
   * Constructor
   * @param document The document to output.
   */
  public ScribblyDocumentWriter(ScribblyDocument document)
  {
    m_document = document;
  }
  
  /**
   * Write to a writer.
   * @param writer
   */
  public void writeTo(Writer writer)
  {
    m_writer = writer;
    m_document.visit(this);
  }
  
  /**
   * Write to a stream.
   * @param stream
   */
  public void writeTo(OutputStream stream)
  {
    m_writer = new OutputStreamWriter(stream);
    m_document.visit(this);
  }
  
  /**
   * Prefix all '[', ']', '{', '}', '@' in the given string with a '@'.
   * @param s The unescaped string
   * @return The escaped string.
   */
  private String escape(String s)
  {
    // @
    String ret = s.replace("@", "@@");
    // [
    ret = ret.replace("[", "@[");
    // ]
    ret = ret.replace("]", "@]");
    // {
    ret = ret.replace("{", "@{");
    // }
    ret = ret.replace("}", "@}");
    //
    return ret;
  }

  @Override
  public void onBeginDocument(ScribblyDocument doc)
  {
    // Nothing to do
  }

  @Override
  public void onEndDocument(ScribblyDocument doc)
  {
    // Nothing to do
  }

  @Override
  public void onBeginCommand(ScribblyCommand command)
  {
    try
    {
      m_writer.write("@" + command.getIdentifier());
      for (Entry<String, String> attr : command.getAttributes())
      {
        m_writer.write("[" + attr.getKey() + ":");
        m_writer.write(escape(attr.getValue()));
        m_writer.write("]");
      }
      m_writer.write("{");
    }
    catch (IOException e)
    {
      throw new RuntimeException("Failed to write beginning of command.", e);
    }
  }

  @Override
  public void onEndCommand(ScribblyCommand command)
  {
    try
    {
      m_writer.write("}");
    }
    catch (IOException e)
    {
      throw new RuntimeException("Failed to write ending of command.", e);
    }
  }

  @Override
  public void onText(ScribblyText text)
  {
    try
    {
      m_writer.write(escape(text.getValue()));
    }
    catch (IOException e)
    {
      throw new RuntimeException("Failed to write text.", e);
    }
  }

}
