package fr.feufochmar.scribbly;

/**
 * Visitor pattern interface for Scribbly documents
 * @author feufochmar
 *
 */
public interface ScribblyVisitor
{
  /**
   * Notify the beginning of a Scribbly document
   * @param doc Document visited.
   */
  void onBeginDocument(ScribblyDocument doc);
  
  /**
   * Notify the end of a Scribbly document
   * @param doc Document visited.
   */
  void onEndDocument(ScribblyDocument doc);
  
  /**
   * Notify the beginning of a Scribbly command.
   * @param command Command visited.
   */
  void onBeginCommand(ScribblyCommand command);
  
  /**
   * Notify the end of a Scribbly command.
   * @param command Command visited.
   */
  void onEndCommand(ScribblyCommand command);
  
  /**
   * Notify a text leaf of a Scribbly document.
   * @param text Leaf visited.
   */
  void onText(ScribblyText text);
}
